

from wpasupplicantconf import WpaSupplicantConf
from .models import WiFiManagerProxy
from .models import WifiNetworkModel


class WPASupplicantProxy(WiFiManagerProxy):
    def __init__(self, cpath='/etc/wpa_supplicant/wpa_supplicant.conf'):
        self._cpath = cpath
        self._config = None
        super(WPASupplicantProxy, self).__init__()

    def _read_config(self):
        self._logger.info("Reading WPA supplicant configuration from {}".format(self._cpath))
        with open(self._cpath, 'r') as f:
            lines = f.readlines()
            self._config = WpaSupplicantConf(lines)

    def _get_configured_networks(self):
        networks = self._config.networks()
        return [WifiNetworkModel(ssid=k, psk=v['psk'])
                for k, v in networks.items()]

    def _write_config(self):
        self._logger.info("Writing WPA supplicant configuration to {}".format(self._cpath))
        with open(self._cpath, 'w') as f:
            self._config.write(f)

    def add_network(self, ssid, psk, **kwargs):
        super(WPASupplicantProxy, self).add_network(ssid, psk, **kwargs)
        psk = '"{}"'.format(psk)
        self._config.add_network(ssid, psk=psk, key_mgmt="WPA-PSK", **kwargs)
        self._write_config()

    def remove_network(self, ssid):
        super(WPASupplicantProxy, self).remove_network(ssid)
        self._config.remove_network(ssid)
        self._write_config()





import logging
from typing import Optional

from typing import List
from fastapi import Body
from fastapi import HTTPException
from fastapi import Depends
from fastapi import APIRouter

from ebs.linuxnode.netconfig.core import app
from ebs.linuxnode.netconfig.core import ActionResultModel
from ebs.linuxnode.netconfig.core import auth_token
from ebs.linuxnode.netconfig import config

from .models import WiFiManagerProxy
from .models import WifiNetworkModel

logger = logging.getLogger(__name__)

_manager_proxy: Optional[WiFiManagerProxy] = None


wifi_router = APIRouter(prefix='/wifi',
                        dependencies=[Depends(auth_token)])


@app.on_event('startup')
async def init():
    global _manager_proxy
    if config.wifi_manager == 'wpa_supplicant':
        from .supplicant import WPASupplicantProxy
        _manager_proxy = WPASupplicantProxy(cpath=config.wpa_supplicant_path)
    elif config.wifi_manager == 'netplan':
        from .netplan import NetplanWifiProxy
        _manager_proxy = NetplanWifiProxy(cpath=config.netplan_path,
                                          device=config.wifi_device)


@wifi_router.get("/networks/show", response_model=List[WifiNetworkModel], status_code=200)
async def show_configured_wifi_networks():
    networks = _manager_proxy.show_networks()
    logger.info("Currently configured networks :\n{}".format(
        "\n".join(["{:20} {}".format(x.ssid, x.psk)
                   for x in networks])))
    return networks


@wifi_router.post("/networks/add", response_model=ActionResultModel, status_code=201)
async def add_wifi_network(network: WifiNetworkModel):
    if _manager_proxy.has_network(network.ssid):
        raise HTTPException(
            status_code=409,
            detail="SSID '{}' already exists. Modify the existing network or remove it and try again."
                   "".format(network.ssid)
        )
    _manager_proxy.add_network(ssid=network.ssid, psk=network.psk)
    return {"result": True}


@wifi_router.post("/networks/remove", response_model=ActionResultModel, status_code=200)
async def remove_wifi_network(ssid: str = Body(...)):
    if not _manager_proxy.has_network(ssid):
        raise HTTPException(
            status_code=416,
            detail="SSID '{}' not recognized. Cannot remove.".format(ssid)
        )
    _manager_proxy.remove_network(ssid=ssid)
    return {"result": True}


@wifi_router.get("/status")
async def wifi_network_status():
    return {"message": "WNS"}


@wifi_router.get("/scan")
async def scan_wifi_networks():
    return {"message": "SN"}


@wifi_router.get("/note")
async def note():
    return _manager_proxy.manager_note





from pydantic import Field
from pydantic import BaseModel

import logging


class WifiNetworkModel(BaseModel):
    ssid: str = Field(None, title="Wifi SSID")
    psk: str = Field(None, title="Wifi PSK (WPA2)")


class WiFiScanProxy(object):
    pass


class WiFiManagerProxy(object):
    _manager_note = ""

    def __init__(self):
        self._logger = logging.getLogger(self.__class__.__name__)
        self._configured_networks = []
        self._read_config()

    @property
    def manager_note(self):
        return self._manager_note

    def _read_config(self):
        raise NotImplementedError

    def _get_configured_networks(self):
        raise NotImplementedError

    @property
    def configured_networks(self):
        if not self._configured_networks:
            self._configured_networks = self._get_configured_networks()
        return self._configured_networks

    @property
    def configured_ssids(self):
        return [x.ssid for x in self.configured_networks]

    def show_networks(self):
        return self.configured_networks

    def has_network(self, ssid):
        return ssid in self.configured_ssids

    def add_network(self, ssid, psk, **kwargs):
        self._logger.info("Adding WiFi network '{}' with psk '{}'".format(ssid, psk))
        self._configured_networks.append(WifiNetworkModel(ssid=ssid, psk=psk))

    def remove_network(self, ssid):
        self._logger.info("Removing WiFi network '{}'".format(ssid))
        for network in self._configured_networks:
            if network.ssid == ssid:
                self._configured_networks.remove(network)





import os
import yaml
from .models import WiFiManagerProxy
from .models import WifiNetworkModel


class NetplanWifiProxy(WiFiManagerProxy):
    _manager_note = "This devices uses the netplan WiFi configuration backend. " \
                    "Only one WiFi network is presently supported. If multiple are " \
                    "configured, the device will not connect to any WiFi networks."

    def __init__(self, cpath='/etc/netplan/01-ebs-netconfig-wifi.conf', device='wlan0'):
        self._cpath = cpath
        self._device = device
        self._config = None
        super(NetplanWifiProxy, self).__init__()

    @property
    def _stub(self):
        return {
            'network': {
                'version': 2,
                'wifis': {
                    self._device: {
                        'access-points': {},
                        'dhcp4': True,
                        'optional': True,
                    }
                }
            }
        }

    def _read_config(self):
        self._logger.info("Reading netplan WiFi configuration from {}".format(self._cpath))
        if not os.path.exists(self._cpath):
            self._logger.info("Writing netplan WiFi configuration stub to {}".format(self._cpath))
            with open(self._cpath, 'w') as f:
                yaml.safe_dump(self._stub, f)
        with open(self._cpath, 'r') as f:
            self._config = yaml.safe_load(f)

    def _get_configured_networks(self):
        return [WifiNetworkModel(ssid=k, psk=v['password'])
                for k, v in self._network_list.items()]

    def _write_config(self):
        self._logger.info("Writing netplan wifi configuration to {}".format(self._cpath))
        with open(self._cpath, 'w') as f:
            yaml.safe_dump(self._config, f)

    @property
    def _network_list(self):
        return self._config['network']['wifis'][self._device]['access-points']

    def add_network(self, ssid, psk, **kwargs):
        super(NetplanWifiProxy, self).add_network(ssid, psk, **kwargs)
        self._network_list[ssid] = {'password': psk}
        self._write_config()

    def remove_network(self, ssid):
        super(NetplanWifiProxy, self).remove_network(ssid)
        self._network_list.pop(ssid)
        self._write_config()
